import os
import uuid
import requests
import json

environment_map = {
   'int-dev-cert': [
     "https://id.ttstage.com/oauth/token",
     "https://pds-int-dev-cert.trade.tt/",
     "https://risk-int-dev-cert.trade.tt/api/1/int-dev-cert/",
     "https://admin-int-dev-cert.trade.tt/",
     "https://algostore-int-dev-cert.trade.tt/",
   ],
   'int-dev-sim': [
     "https://id.ttstage.com/oauth/token",
     "https://pds-int-dev-sim.trade.tt/",
     "https://risk-int-dev-sim.trade.tt/api/1/int-dev-sim/",
     "https://admin-int-dev-sim.trade.tt/",
     "https://algostore-int-dev-sim.trade.tt/",
   ],
   'int-stage-cert': [
     "https://id.ttstage.com/oauth/token",
     "https://pds-int-stage-cert.trade.tt/",
     "https://risk-int-stage-cert.trade.tt/api/1/int-stage-cert/",
     "https://admin-int-stage-cert.trade.tt/",
     "https://algostore-int-stage-cert.trade.tt/",
   ],
   'int-stage-sim': [
     "https://id.ttstage.com/oauth/token",
     "https://pds-int-stage-sim.trade.tt/",
     "https://risk-int-stage-sim.trade.tt/api/1/int-stage-sim/",
     "https://admin-int-stage-sim.trade.tt/",
     "https://algostore-int-stage-sim.trade.tt/",
   ],
   'ext-uat-cert': [
     "https://id-uat.tradingtechnologies.com/oauth/token",
     "https://pds-ext-uat-cert.trade.tt/",
     "https://risk-ext-uat-cert.trade.tt/api/1/ext-uat-cert/",
     "https://admin-ext-uat-cert.trade.tt/",
     "https://algostore-ext-uat-cert.trade.tt/",
   ],
   'ext-prod-sim': [
     "https://id.tradingtechnologies.com/oauth/token",
     "https://pds-ext-prod-sim.trade.tt/",
     "https://risk-ext-prod-sim.trade.tt/api/1/ext-prod-sim/",
     "https://admin-ext-prod-sim.trade.tt/",
     "https://algostore-ext-prod-sim.trade.tt/",
   ],
   'ext-prod-live': [
     "https://id.tradingtechnologies.com/oauth/token",
     "https://pds-ext-prod-live.trade.tt/",
     "https://risk-ext-prod-live.trade.tt/api/1/ext-prod-live/",
     "https://admin-ext-prod-live.trade.tt/",
     "https://algostore-ext-prod-live.trade.tt/"
   ],
}


def get_auth_url():
    env = get_environment()
    if env not in environment_map:
        return ""
    return environment_map[env][0]

def get_user_url():
    env = get_environment()
    if env not in environment_map:
        return ""
    return environment_map[env][2]

def get_admin_url():
    env = get_environment()
    if env not in environment_map:
        return ""
    return environment_map[env][3]

def get_algostore_url():
    env = get_environment()
    if env not in environment_map:
        return ""
    return environment_map[env][4]

def get_pds_url():
    env = get_environment()
    if env not in environment_map:
        return ""
    return environment_map[env][1]

def get_access_token(appkey, onBehalfOf):
    requestId = "ttsdkalgo-script--{0}".format(uuid.uuid4())
    url = get_auth_url() + "?request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    print("Requesting token for {0} with requestId={1}".format(onBehalfOf, requestId))
    login_payload = "grant_type=user_app&app_key=" + appkey
    r = requests.post(url=get_auth_url(), data=login_payload)
    if not r.ok:
        raise Exception("Login failed")
    jsonResp = json.loads(r.content)
    return jsonResp['access_token']

def get_user_id(token, onBehalfOf):
    requestId = "ttsdkalgo-script--{0}".format(uuid.uuid4())
    url = get_user_url() + "users/me?request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    print("Requesting user for {0} with requestId={1}".format(onBehalfOf, requestId))
    headers = {"Authorization": "Bearer " + token}
    r = requests.get(url=url, headers=headers)
    if not r.ok:
        raise Exception("Login failed")
    jsonResp = json.loads(r.content)
    return jsonResp['aResult'][0]['oInfo']['iUserId']

def get_person_id(email, token, onBehalfOf):
    requestId = "ttsdkalgo-script--{0}".format(uuid.uuid4())
    print("Requesting person {0} for {1} with requestId={2}".format(email, onBehalfOf, requestId))
    url = get_admin_url() + "persons/" + email + "?request_id=" + requestId + "&onbehalf_request_id=" + onBehalfOf
    headers = {"Authorization": "Bearer " + token}
    r = requests.get(url=url, headers=headers)
    if not r.ok:
        raise Exception("Login failed")
    jsonResp = json.loads(r.content)
    return jsonResp['aResult'][0]['oInfo']['iPersonId']

def get_environment():
    env = ""
    if hasattr(environment, "content"):
        env = environment.content

    CONST__CHEF_ENVIRONMENT = 'CHEF_ENVIRONMENT'
    if env.strip() == "":
        if CONST__CHEF_ENVIRONMENT in os.environ:
            env = os.environ[CONST__CHEF_ENVIRONMENT]

    if env.strip() == "":
        env = "ext-prod-sim"

    return env


def environment(env):
    if not hasattr(environment, "content"):
        environment.content = ""
    environment.content = env
    print("Target environment is '{0}'".format(get_environment()))


